@@ -6,31 +6,26 @@ To run the server type::
     
 Open a new shell and launch python and type::
 
-    >>> from pulsar.http import rpc
+    >>> from pulsar.apps import rpc
     >>> p = rpc.JsonProxy('http://localhost:8060')
     >>> p.ping()
     'pong'
+    >>> p.functions_list()
+    >>> ...
     >>> p.calc.add(3,4)
     7.0
     >>>
     
 '''
-try:
-    from penv import pulsar
-except ImportError:
-    import pulsar
-    
-from pulsar.http import rpc
+from pulsar.apps import rpc, wsgi
 
 
-class Root(rpc.JSONRPC):
-    
-    def rpc_ping(self, request):
-        return 'pong'
-    
-    def rpc_server_info(self, request, full = False):
-        worker = request.environ['pulsar.worker']
-        return worker.proxy.info(worker.arbiter, full = full)
+def divide(a,b):
+    '''Divide two numbers'''
+    return float(a) / float(b)
+
+class Root(rpc.PulsarServerCommands):
+    pass
         
     
 class Calculator(rpc.JSONRPC):
@@ -44,13 +39,11 @@ class Calculator(rpc.JSONRPC):
     def rpc_multiply(self, request, a, b):
         return float(a) * float(b)
     
-    def rpc_divide(self, request, a, b):
-        return float(a) / float(b)
+    rpc_divide = rpc.FromApi(divide)
 
 
 def server(**params):
-    root = Root().putSubHandler('calc',Calculator())
-    wsgi = pulsar.require('wsgi')
+    root = rpc.RpcMiddleware(Root().putSubHandler('calc',Calculator()))
     return wsgi.createServer(callable = root, **params)
 
 
