@@ -1,5 +1,5 @@
 '''Concurrent server and message queues'''
-VERSION = (0, 1, 0)
+VERSION = (0, 2, 0)
 
 def get_version():
     return '.'.join(map(str,VERSION))
@@ -28,17 +28,22 @@ from .utils.log import *
 DEFAULT_PORT = 8060
 SERVER_SOFTWARE = "python-{0}/{1}".format(SERVER_NAME,get_version())
 
+class NOT_DONE(object):
+    pass
+
+class CLEAR_ERRORS(object):
+    pass
+
 from .utils.exceptions import *
-from .utils import system
 from .utils.sock import *
-from .utils.py2py3 import ispy3k, to_string, is_string
+from .utils import system
+from .utils.py2py3 import ispy3k, to_string, is_string, native_str,\
+                            to_bytestring
 platform = system.platform
 
 from .utils.config import *
 from .async import *
-from .workers.base import *
-from .workers.monitor import *
-from .apps.base import Application, require
-from .apps import test
+from .apps import *
+
 
 
