@@ -1,5 +1,6 @@
 import pulsar
 from pulsar import to_string
+from pulsar.utils.importer import module_attribute
 
 
 def validate_list(val):
@@ -18,6 +19,7 @@ def validate_list(val):
 
 
 class TaskPath(pulsar.Setting):
+    app = 'tasks'
     name = "tasks_path"
     section = "Task Consumer"
     meta = "STRING"
@@ -27,3 +29,16 @@ class TaskPath(pulsar.Setting):
     desc = """\
         List of python dotted paths where tasks are located.
         """
+
+
+class TaskQueueFactory(pulsar.Setting):
+    #app = 'tasks'
+    name = "task_queue_factory"
+    section = "Task Consumer"
+    flags = ["-q", "--task-queue"]
+    default = "pulsar.Queue"
+    desc = """The task queue factory to use."""
+    
+    def get(self):
+        return module_attribute(self.value)
+        return self.value
\ No newline at end of file
