@@ -1,5 +1,6 @@
 '''\
-Configuration utilities. originally from gunicorn_, adapted and modified for pulsar.
+Configuration utilities. originally from gunicorn_,
+adapted and modified for pulsar.
 
 Original Gunicorn Licence
 
@@ -10,27 +11,31 @@ See the NOTICE for more information.
 '''
 import copy
 import inspect
-import optparse
+import argparse
 import os
 import textwrap
 import types
 
-from pulsar import __version__, SERVER_NAME, DEFAULT_PORT
+from pulsar import __version__, SERVER_NAME
 from pulsar.utils import system
 from pulsar.utils.py2py3 import *
 
 
 __all__ = ['Config',
-           'DummyConfig',
            'Setting',
+           'ordered_settings',
            'validate_string',
            'validate_callable',
            'validate_bool',
+           'validate_list',
            'validate_pos_int',
-           'make_settings']
+           'make_settings',
+           'make_options']
 
+    
+KNOWN_SETTINGS = {}
+KNOWN_SETTINGS_ORDER = []
 
-KNOWN_SETTINGS = []
 
 def def_start_server(server):
     pass
@@ -61,45 +66,94 @@ def wrap_method(func):
         return func(*args, **kwargs)
     return _wrapped
 
+    
+def ordered_settings():
+    for name in KNOWN_SETTINGS_ORDER:
+        yield KNOWN_SETTINGS[name]
+        
+        
+def make_settings(app = None, include=None, exclude=None):
+    '''Creates a dictionary of available settings for a given
+application *app*.
 
-def make_settings(ignore=None):
+:parameter app: Optional application name.
+:parameter include: Optional list of settings to include.
+:parameter app: Optional list of settings to exclude.
+:rtype: dictionary of :class:`pulsar.Setting` instances.'''
     settings = {}
-    ignore = ignore or ()
-    for s in KNOWN_SETTINGS:
+    exclude = exclude or ()
+    for s in ordered_settings():
         setting = s()
-        if setting.name in ignore:
+        if setting.app and setting.app != app:
+            continue
+        if include and setting.name not in include and not setting.app:
+            continue
+        if setting.name in exclude:
             continue
         settings[setting.name] = setting.copy()
     return settings
 
 
-class DummyConfig(object):
+def make_options():
+    g_settings = make_settings(exclude=('version',))
 
-    def __getstate__(self):
-        return self.__dict__.copy()
+    keys = g_settings.keys()
+    def sorter(k):
+        return (g_settings[k].section, g_settings[k].order)
+
+    opts = []
     
-    def __setstate__(self, state):
-        self.__dict__ = state
-            
-    def __getattr__(self, name):
-        return None
+    for k in keys:
+        setting = g_settings[k]
+        if not setting.flags:
+            continue
+
+        args = tuple(setting.flags)
+
+        kwargs = {
+            "dest": setting.name,
+            "metavar": setting.meta or None,
+            "action": setting.action or "store",
+            "type": setting.type or "string",
+            "default": None,
+            "help": "%s [%s]" % (setting.short, setting.default)
+        }
+        if kwargs["action"] != "store":
+            kwargs.pop("type")
+
+        opts.append(optparse.make_option(*args, **kwargs))
+
+    return tuple(opts)
 
 
 class Config(object):
-        
-    def __init__(self, usage=None):
-        self.settings = make_settings()
-        self.usage = usage
+    '''Dictionary containing :class:`Setting` parameters for
+fine tuning pulsar servers. It provides easy access to :attr:`Setting.value`
+attribute by exposing the :attr:`Setting.name` as attribute.
+
+.. attribute:: settings
+
+    Dictionary of all :class:`Settings` instances available. The
+    keys are given by the :attr:`Setting.name` attribute.
+'''
+    def __init__(self, description = None, epilog = None,
+                 app = None, include=None, exclude = None):
+        self.settings = make_settings(app,include,exclude)
+        self.description = description or 'Pulsar server'
+        self.epilog = epilog or 'Have fun!'
         
     def __getstate__(self):
         return self.__dict__.copy()
     
     def __setstate__(self, state):
         self.__dict__['settings'] = state['settings']
-        self.__dict__['usage'] = state['usage']
+        self.__dict__['description'] = state['description']
+        self.__dict__['epilog'] = state['epilog']
     
     def __getattr__(self, name):
         if name not in self.settings:
+            if name in KNOWN_SETTINGS:
+                return None
             raise AttributeError("No configuration setting for: %s" % name)
         return self.settings[name].get()
     
@@ -114,17 +168,25 @@ class Config(object):
         self.settings[name].set(value)
 
     def parser(self):
-        kwargs = {
-            "usage": self.usage,
-            "version": __version__
-        }
-        parser = optparse.OptionParser(**kwargs)
+        '''Create the argparser_ for this configuration by adding all
+settings via the :meth:`Setting.add_argument`.
 
-        keys = self.settings.keys()
-        sorter = lambda x: (self.settings[x].section, self.settings[x].order)
+:rtype: an instance of :class:`argparse.ArgumentParser`.
         
-        for k in sorted(keys,key=sorter):
-            self.settings[k].add_option(parser)
+.. _argparser: http://docs.python.org/dev/library/argparse.html
+'''
+        kwargs = {
+            "description": self.description,
+            "epilog": self.epilog
+        }
+        parser = argparse.ArgumentParser(**kwargs)
+        parser.add_argument('--version',
+                            action='version',
+                            version = __version__)
+        setts = self.settings
+        sorter = lambda x: (setts[x].section, setts[x].order)
+        for k in sorted(setts,key=sorter):
+            setts[k].add_argument(parser)
         return parser
 
     @property
@@ -134,36 +196,45 @@ class Config(object):
         if hasattr(worker_class, "setup_class"):
             worker_class.setup_class()
         return worker_class
-
+    
     @property
     def workers(self):
         return self.settings['workers'].get()
 
     @property
     def address(self):
-        bind = self.settings['bind'].get()
-        return system.parse_address(to_bytestring(bind))
+        bind = self.settings['bind']
+        if bind:
+            return system.parse_address(to_bytestring(bind.get()))
         
     @property
     def uid(self):
-        user = self.settings['user'].get()
-        return system.get_uid(user)
+        user = self.settings.get('user')
+        if user:
+            return system.get_uid(user.get())
         
     @property
     def gid(self):
-        group = self.settings['group'].get()
-        return system.get_gid(group)
+        group = self.settings.get('group')
+        if group:
+            return system.get_gid(group.get())
         
     @property
     def proc_name(self):
-        pn = self.settings['proc_name'].get()
+        pn = self.settings.get('proc_name')
+        if pn:
+            pn = pn.get()
         if pn is not None:
             return pn
         else:
-            return self.settings['default_proc_name'].get()
+            pn = self.settings.get('default_proc_name')
+            if pn:
+                return pn.get()
             
             
 class SettingMeta(type):
+    '''A metaclass which collects all setting classes and put them
+in the global ``KNOWN_SETTINGS`` list.'''
     def __new__(cls, name, bases, attrs):
         super_new = super(SettingMeta, cls).__new__
         parents = [b for b in bases if isinstance(b, SettingMeta)]
@@ -171,11 +242,19 @@ class SettingMeta(type):
             return super_new(cls, name, bases, attrs)            
     
         attrs["order"] = len(KNOWN_SETTINGS)
-        attrs["validator"] = wrap_method(attrs["validator"])
+        val = attrs.get("validator")
+        attrs["validator"] = wrap_method(val) if val else None
         
         new_class = super_new(cls, name, bases, attrs)
         new_class.fmt_desc(attrs['desc'] or '')
-        KNOWN_SETTINGS.append(new_class)
+        if not new_class.name:
+            new_class.name = new_class.__name__.lower()
+        if new_class.name in KNOWN_SETTINGS_ORDER:
+            old_class = KNOWN_SETTINGS.pop(new_class.name)
+            new_class.order = old_class.order
+        else:
+            KNOWN_SETTINGS_ORDER.append(new_class.name)
+        KNOWN_SETTINGS[new_class.name] = new_class
         return new_class
 
     def fmt_desc(cls, desc):
@@ -190,38 +269,72 @@ BaseSettings =  SettingMeta('BaseSettings', (object, ), {})
 
 
 class Setting(BaseSettings):
-    virtual = True    
+    '''A configuration parameter for pulsar. Parameters can be specified
+on the command line or on a config file.'''
+    virtual = True
+    '''If set to ``True`` the settings won't be loaded and it can be only used
+as base class for other settings.'''
     name = None
+    '''The unique name used to access this setting.'''
+    nargs = None
+    '''For positional arguments. Same usage as argparse.'''
+    app = None
+    '''Setting for a specific :class:`Application`.'''
     value = None
+    '''The actual value for this setting.'''
     section = None
-    cli = None
+    '''Setting section, used for creating documentation.'''
+    flags = None
+    '''List of options strings, e.g. ``[-f, --foo]``.'''
+    choices = None
+    '''Restrict the argument to the choices provided.'''
     validator = None
+    '''Validator for this setting. It provided it must be a fucntion
+accepting one positional argument, the value to validate.'''
     type = None
     meta = None
     action = None
     default = None
+    '''Default value'''
     short = None
     desc = None
     
     def __init__(self):
         if self.default is not None:
-            self.set(self.default)    
+            self.set(self.default)
+        self.short = self.short or self.desc
+        self.desc = self.desc or self.short
+        if self.app and not self.section:
+            self.section = self.app
+            
+    def __str__(self):
+        return '{0} ({1})'.format(self.name,self.value)
+    __repr__ = __str__
         
-    def add_option(self, parser):
-        if not self.cli:
+    def add_argument(self, parser):
+        '''Add itself to the argparser.'''
+        kwargs = {'nargs':self.nargs}
+        if self.type and self.type != 'string':
+            kwargs["type"] = self.type
+            
+        if self.flags:
+            args = tuple(self.flags)
+            kwargs.update({"dest": self.name,
+                           "action": self.action or "store",
+                           "default": None,
+                           "help": "%s [%s]" % (self.short, self.default)})
+            if kwargs["action"] != "store":
+                kwargs.pop("type",None)
+                kwargs.pop("nargs",None)
+        elif self.nargs and self.name:
+            args = (self.name,)
+            kwargs.update({'metavar': self.meta or None,
+                           'help': self.short})
+        else:
+            # Not added to argparser
             return
-        args = tuple(self.cli)
-        kwargs = {
-            "dest": self.name,
-            "metavar": self.meta or None,
-            "action": self.action or "store",
-            "type": self.type or "string",
-            "default": None,
-            "help": "%s [%s]" % (self.short, self.default)
-        }
-        if kwargs["action"] != "store":
-            kwargs.pop("type")
-        parser.add_option(*args, **kwargs)
+        
+        parser.add_argument(*args, **kwargs)
     
     def copy(self):
         return copy.copy(self)
@@ -230,8 +343,9 @@ class Setting(BaseSettings):
         return self.value
     
     def set(self, val):
-        assert hasattr(self.validator,'__call__'), "Invalid validator: %s" % self.name
-        self.value = self.validator(val)
+        if hasattr(self.validator,'__call__'):
+            val = self.validator(val)
+        self.value = val
 
 
 def validate_bool(val):
@@ -266,6 +380,18 @@ def validate_string(val):
     return to_string(val).strip()
 
 
+def validate_list(val):
+    if val and not isinstance(val,list):
+        raise TypeError("Not a list: %s" % val)
+    return val
+
+
+def validate_dict(val):
+    if val and not isinstance(val,dict):
+        raise TypeError("Not a dictionary: %s" % val)
+    return val
+
+
 def validate_callable(arity):
     def _validate_callable(val):
         if not hasattr(val,'__call__'):
@@ -285,7 +411,7 @@ def validate_callable(arity):
 class ConfigFile(Setting):
     name = "config"
     section = "Config File"
-    cli = ["-c", "--config"]
+    flags = ["-c", "--config"]
     meta = "FILE"
     validator = validate_string
     default = 'config.py'
@@ -295,117 +421,39 @@ class ConfigFile(Setting):
         Only has an effect when specified on the command line or as part of an
         application specific configuration.    
         """
-
-
-class Bind(Setting):
-    name = "bind"
-    section = "Server Socket"
-    cli = ["-b", "--bind"]
-    meta = "ADDRESS"
-    validator = validate_string
-    default = "127.0.0.1:{0}".format(DEFAULT_PORT)
-    desc = """\
-        The socket to bind.
         
-        A string of the form: 'HOST', 'HOST:PORT', 'unix:PATH'. An IP is a valid
-        HOST.
-        """
-        
-        
-class Backlog(Setting):
-    name = "backlog"
-    section = "Server Socket"
-    cli = ["--backlog"]
-    meta = "INT"
-    validator = validate_pos_int
-    type = "int"
-    default = 2048
-    desc = """\
-        The maximum number of pending connections.    
-        
-        This refers to the number of clients that can be waiting to be served.
-        Exceeding this number results in the client getting an error when
-        attempting to connect. It should only affect servers under significant
-        load.
-        
-        Must be a positive integer. Generally set in the 64-2048 range.    
-        """
-
 
 class Workers(Setting):
     name = "workers"
     section = "Worker Processes"
-    cli = ["-w", "--workers"]
-    meta = "INT"
+    flags = ["-w", "--workers"]
     validator = validate_pos_int
-    type = "int"
+    type = int
     default = 1
     desc = """\
         The number of worker process for handling requests.
         
-        A positive integer generally in the 2-4 x $(NUM_CORES) range. You'll
-        want to vary this a bit to find the best for your particular
-        application's work load.
-        """
-
-
-class WorkerClass(Setting):
-    name = "worker_class"
-    section = "Worker Processes"
-    cli = ["-k", "--worker-class"]
-    meta = "STRING"
-    validator = validate_string
-    default = "http"
-    desc = """\
-        The type of workers to use.
-        
-        A string referring to one of the following bundled classes:
-        
-        * ``sync``
-        * ``eventlet`` - Requires eventlet >= 0.9.7
-        * ``gevent``   - Requires gevent >= 0.12.2 (?)
-        * ``tornado``  - Requires tornado >= 0.2
-        
-        Optionally, you can provide your own worker by giving pulsar a
-        MODULE:CLASS pair where CLASS is a subclass of
-        pulsar.Worker.
-        """
+If you are using a multi-process concurrency, a number in the
+the 2-4 x $(NUM_CORES) range should be good. If you are using threads this
+number can be higher."""
 
 
 class Concurrency(Setting):
     name = "concurrency"
     section = "Worker Processes"
-    cli = ["--concurrency"]
-    meta = "STRING"
-    validator = validate_string
+    flags = ["--concurrency"]
     default = "process"
     desc = """\
-        The type of concurrency to use: process or thread.
+        The type of concurrency to use: ``process`` or ``thread``.
         """
         
 
-class WorkerConnections(Setting):
-    name = "worker_connections"
-    section = "Worker Processes"
-    cli = ["--worker-connections"]
-    meta = "INT"
-    validator = validate_pos_int
-    type = "int"
-    default = 1000
-    desc = """\
-        The maximum number of simultaneous clients.
-        
-        This setting only affects the Eventlet and Gevent worker types.
-        """
-
-
 class MaxRequests(Setting):
     name = "max_requests"
     section = "Worker Processes"
-    cli = ["--max-requests"]
-    meta = "INT"
+    flags = ["--max-requests"]
     validator = validate_pos_int
-    type = "int"
+    type = int
     default = 0
     desc = """\
         The maximum number of requests a worker will process before restarting.
@@ -422,10 +470,9 @@ class MaxRequests(Setting):
 class Timeout(Setting):
     name = "timeout"
     section = "Worker Processes"
-    cli = ["-t", "--timeout"]
-    meta = "INT"
+    flags = ["-t", "--timeout"]
     validator = validate_pos_int
-    type = "int"
+    type = int
     default = 30
     desc = """\
         Workers silent for more than this many seconds are killed and restarted.
@@ -437,25 +484,20 @@ class Timeout(Setting):
         """
 
 
-class Keepalive(Setting):
-    name = "keepalive"
-    section = "Worker Processes"
-    cli = ["--keep-alive"]
-    meta = "INT"
-    validator = validate_pos_int
-    type = "int"
-    default = 2
+class HttpProxyServer(Setting):
+    name = "http_proxy"
+    section = "Http Client"
+    flags = ["--http-proxy"]
+    default = ''
     desc = """\
-        The number of seconds to wait for requests on a Keep-Alive connection.
-        
-        Generally set in the 1-5 seconds range.    
+        The HTTP proxy server to use with HttpClient.    
         """
 
 
 class Debug(Setting):
     name = "debug"
     section = "Debugging"
-    cli = ["--debug"]
+    flags = ["--debug"]
     validator = validate_bool
     action = "store_true"
     default = False
@@ -465,43 +507,12 @@ class Debug(Setting):
         This limits the number of worker processes to 1 and changes some error
         handling that's sent to clients.
         """
-
-
-class Spew(Setting):
-    name = "spew"
-    section = "Debugging"
-    cli = ["--spew"]
-    validator = validate_bool
-    action = "store_true"
-    default = False
-    desc = """\
-        Install a trace function that spews every line executed by the server.
         
-        This is the nuclear option.    
-        """
-
-
-class PreloadApp(Setting):
-    name = "preload_app"
-    section = "Server Mechanics"
-    cli = ["--preload"]
-    validator = validate_bool
-    action = "store_true"
-    default = False
-    desc = """\
-        Load application code before the worker processes are forked.
-        
-        By preloading an application you can save some RAM resources as well as
-        speed up server boot times. Although, if you defer application loading
-        to each worker process, you can reload your application code easily by
-        restarting workers.
-        """
-
 
 class Daemon(Setting):
     name = "daemon"
     section = "Server Mechanics"
-    cli = ["-D", "--daemon"]
+    flags = ["-D", "--daemon"]
     validator = validate_bool
     action = "store_true"
     default = False
@@ -516,7 +527,7 @@ class Daemon(Setting):
 class Pidfile(Setting):
     name = "pidfile"
     section = "Server Mechanics"
-    cli = ["-p", "--pid"]
+    flags = ["-p", "--pid"]
     meta = "FILE"
     validator = validate_string
     default = None
@@ -530,7 +541,7 @@ class Pidfile(Setting):
 class User(Setting):
     name = "user"
     section = "Server Mechanics"
-    cli = ["-u", "--user"]
+    flags = ["-u", "--user"]
     meta = "USER"
     validator = validate_string
     default = None
@@ -546,7 +557,7 @@ class User(Setting):
 class Group(Setting):
     name = "group"
     section = "Server Mechanics"
-    cli = ["-g", "--group"]
+    flags = ["-g", "--group"]
     meta = "GROUP"
     validator = validate_string
     default = None
@@ -562,10 +573,9 @@ class Group(Setting):
 class Umask(Setting):
     name = "umask"
     section = "Server Mechanics"
-    cli = ["-m", "--umask"]
-    meta = "INT"
+    flags = ["-m", "--umask"]
     validator = validate_pos_int
-    type = "int"
+    type = int
     default = 0
     desc = """\
         A bit mask for the file mode on files written by Gunicorn.
@@ -593,60 +603,52 @@ class TmpUploadDir(Setting):
         workers. If not specified, Gunicorn will choose a system generated
         temporary directory.
         """
-        
-
-class Httplib(Setting):
-    name = "httplib"
-    section = "Process Naming"
-    cli = ["--http"]
-    meta = "STRING"
-    validator = validate_string
-    default = 'gunicorn'
-    desc = """\
-        HTTP library used by server.
-        
-        It defaults to 'gunicorn'.
-        """
-
-
-class Logfile(Setting):
-    name = "logfile"
-    section = "Logging"
-    cli = ["--log-file"]
-    meta = "FILE"
-    validator = validate_string
-    default = "-"
-    desc = """\
-        The log file to write to.
-        
-        "-" means log to stdout.
-        """
 
 
 class Loglevel(Setting):
     name = "loglevel"
     section = "Logging"
-    cli = ["--log-level"]
+    flags = ["--log-level"]
     meta = "LEVEL"
     validator = validate_string
     default = "info"
-    desc = """\
-        The granularity of log outputs.
+    desc = """The granularity of log outputs.
         
-        Valid level names are:
-        
-        * debug
-        * info
-        * warning
-        * error
-        * critical
-        """
+Valid level names are:
+
+ * debug
+ * info
+ * warning
+ * error
+ * critical
+ """
 
 
+class LogEvery(Setting):
+    name = "logevery"
+    section = "Logging"
+    flags = ["--log-every"]
+    validator = validate_pos_int
+    default = 0
+    desc = """Log information every n seconds"""
+    
+    
+class LogConfig(Setting):
+    name = "logconfig"
+    section = "Logging"
+    default = {}
+    validator = validate_dict
+    desc = '''
+    The logging configuration dictionary.
+    
+    This settings can only be specified on a config file
+    '''
+    
+    
 class Procname(Setting):
     name = "proc_name"
     section = "Process Naming"
-    cli = ["-n", "--name"]
+    flags = ["-n", "--name"]
     meta = "STRING"
     validator = validate_string
     default = None
