@@ -4,9 +4,18 @@ import resource
 import grp
 import pwd
 import signal
+from multiprocessing import Pipe
 
 from .base import *
 
+__all__ = ['IOpoll',
+           'close_on_exec',
+           'Waker',
+           'daemonize',
+           'SIGQUIT',
+           'get_uid',
+           'get_gid']
+
 
 import select
 if hasattr(select,'epoll'):
@@ -14,7 +23,7 @@ if hasattr(select,'epoll'):
 else:
     IOpoll = IOselect
 
-
+# The standard signal quit
 SIGQUIT = signal.SIGQUIT
 
 def get_parent_id():
@@ -29,14 +38,11 @@ def chown(path, uid, gid):
         
         
 def close_on_exec(fd):
-    try:
-        flags = fcntl.fcntl(fd, fcntl.F_GETFD)
-        fcntl.fcntl(fd, fcntl.F_SETFD, flags | fcntl.FD_CLOEXEC)
-    except:
-        pass
+    flags = fcntl.fcntl(fd, fcntl.F_GETFD)
+    fcntl.fcntl(fd, fcntl.F_SETFD, flags | fcntl.FD_CLOEXEC)
     
     
-def __set_non_blocking(fd):
+def _set_non_blocking(fd):
     flags = fcntl.fcntl(fd, fcntl.F_GETFL) | os.O_NONBLOCK
     fcntl.fcntl(fd, fcntl.F_SETFL, flags)
     
@@ -62,6 +68,37 @@ def setpgrp():
     os.setpgrp()
 
 
+class Waker(object):
+    
+    def __init__(self):
+        r, w = Pipe(duplex = False)
+        _set_non_blocking(r.fileno())
+        _set_non_blocking(w.fileno())
+        close_on_exec(r.fileno())
+        close_on_exec(w.fileno())
+        self._writer = w
+        self._reader = r
+        
+    def __str__(self):
+        return 'Pipe waker {0}'.format(self.fileno())
+        
+    def fileno(self):
+        return self._reader.fileno()
+    
+    def wake(self):
+        try:
+            self._writer.send(b'x')
+        except IOError:
+            pass
+
+    def consume(self):
+        r = self._reader
+        try:
+             while r.poll():
+                 r.recv()
+        except IOError:
+            pass
+
     
 def daemonize():
     """\
